<?php
/**
 * Wicked Page class for most pages.
 *
 * $Horde: wicked/lib/Page/StandardPage.php,v 1.74 2008-02-29 19:18:05 chuck Exp $
 *
 * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Tyler Colbert <tyler@colberts.us>
 * @package Wicked
 */
class StandardPage extends Page {

    /**
     * Display modes supported by this page.
     *
     * @var array
     */
    var $supportedModes = array(
        WICKED_MODE_DISPLAY => true,
        WICKED_MODE_EDIT => true,
        WICKED_MODE_REMOVE => true,
        WICKED_MODE_HISTORY => true,
        WICKED_MODE_DIFF => true,
        WICKED_MODE_LOCKING => true);

    /**
     * Constructs a standard page class to represent a wiki page.
     *
     * @param string $pagename The name of the page to represent.
     */
    function StandardPage($pagename)
    {
        if (is_array($pagename)) {
            $this->_page = $pagename;
            return;
        }

        global $wicked, $notification, $perms;
        $page = $wicked->retrieveByName($pagename);

        // Make sure 'wicked' permission exists. Set reasonable defaults if
        // necessary.
        if (!$perms->exists('wicked')) {
            $perm = &$perms->newPermission('wicked');
            $perm->addGuestPermission(PERMS_SHOW | PERMS_READ);
            $perm->addDefaultPermission(PERMS_SHOW | PERMS_READ | PERMS_EDIT | PERMS_DELETE);
        }

        // Make sure 'wicked:pages' exists. Copy from 'wicked' if it does not
        // exist.
        if (!$perms->exists('wicked:pages')) {
            $perm = &$perms->newPermission('wicked:pages');
            $copyFrom = &$perms->getPermission('wicked');
            $perm->addGuestPermission($copyFrom->getGuestPermissions());
            $perm->addDefaultPermission($copyFrom->getDefaultPermissions());
            $perm->addCreatorPermission($copyFrom->getCreatorPermissions());
            foreach ($copyFrom->getUserPermissions() as $user => $uperm) {
                $perm->addUserPermission($user, $uperm, false);
            }
            foreach ($copyFrom->getGroupPermissions() as $group => $gperm) {
                $perm->addGroupPermission($group, $gperm, false);
            }
        }

        // If we can't load $pagename, see if there's default data for it.
        if (is_a($page, 'PEAR_Error')) {
            $pagefile = WICKED_BASE . '/scripts/data/' . basename($pagename);
            if ($pagename == basename($pagename) &&
                file_exists($pagefile)) {
                $text = file_get_contents($pagefile);
                $result = $wicked->newPage($pagename, $text);
                if (!is_a($result, 'PEAR_Error')) {
                    $page = $wicked->retrieveByName($pagename);
                    if (is_a($page, 'PEAR_Error')) {
                        $notification->push(sprintf(_("Unable to create %s"), $pagename), 'horde.error');
                    }
                }
            }
        }

        if (is_a($page, 'PEAR_Error')) {
            if ($pagename == 'WikiHome') {
                $notification->push(_("Unable to create WikiHome. The wiki is not configured."), 'horde.error');
            }
            $this->_page = array();
        } else {
            $this->_page = $page;
        }
    }

    function displayContents($isBlock)
    {
        global $wicked;

        $wiki = &$this->getProcessor();
        $text = $wiki->transform($this->getText());
        $attachments = array();

        if (!$isBlock) {
            $pageId = $wicked->getPageId($this->pageName());
            if (!is_a($pageId, 'PEAR_Error')) {
                $attachments = $wicked->getAttachedFiles($wicked->getPageId($this->pageName()));
                if (is_a($attachments, 'PEAR_Error')) {
                    $attachments = array();
                }
            }

            if (count($attachments)) {
                global $mime_drivers, $mime_drivers_map;
                require_once 'Horde/MIME/Magic.php';
                require_once 'Horde/MIME/Viewer.php';
                if (is_callable(array('Horde', 'loadConfiguration'))) {
                    $result = Horde::loadConfiguration('mime_drivers.php', array('mime_drivers', 'mime_drivers_map'), 'horde');
                    extract($result);
                } else {
                    require HORDE_BASE . '/config/mime_drivers.php';
                }
            }
        }

        ob_start();
        require WICKED_TEMPLATES . '/display/standard.inc';
        $result = ob_get_contents();
        ob_end_clean();
        return $result;
    }

    /**
     * Renders this page in History mode.
     */
    function history()
    {
        global $wicked, $notification;
        require_once WICKED_BASE . '/lib/Page/StandardPage/StdHistoryPage.php';

        $summaries = $wicked->getHistory($this->pageName());
        if (is_a($summaries, 'PEAR_Error')) {
            $notification->push('Error retrieving histories : ' . $summaries->getMessage(), 'horde.error');
            return $summaries;
        }

        // Header.
        $show_restore = !$this->isLocked();
        $allow_diff = true;
        $show_edit = true;
        require WICKED_TEMPLATES . '/history/header.inc';
        $style = 'text';

        // First item is this page.
        $show_restore = false;
        $page = $this;
        $i = 0;
        require WICKED_TEMPLATES . '/history/summary.inc';

        // Now the rest of the histories.
        $show_restore = !$this->isLocked();
        $show_edit = false;
        foreach ($summaries as $page) {
            $i++;
            $page = new StdHistoryPage($page);
            require WICKED_TEMPLATES . '/history/summary.inc';
        }

        // Footer.
        require WICKED_TEMPLATES . '/history/footer.inc';
    }

    function isLocked()
    {
        return !empty($this->_page['page_locked']);
    }

    function lock()
    {
        if ($GLOBALS['wicked']->lock($this->pageName(), true)) {
            $this->_page['page_locked'] = true;
        }
    }

    function unlock()
    {
        if ($GLOBALS['wicked']->lock($this->pageName(), false)) {
            $this->_page['page_locked'] = false;
        }
    }

    function updateText($newtext, $changelog, $minorchange)
    {
        $version = $this->version();
        $result = $GLOBALS['wicked']->updateText($this->pageName(), $newtext,
                                                 $changelog, $minorchange);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $url = Wicked::url($this->pageName(), true, -1);
        $new_page = $this->getPage($this->pageName());

        $message = "Modified page: $url\n"
            . 'New Revision:  ' . $new_page->version() . "\n"
            . ($changelog ? 'Change log:  ' . $changelog . "\n" : '')
            . "\n"
            . $new_page->getDiff($version);
        Wicked::mail($message,
                     array('Subject' => '[' . $GLOBALS['registry']->get('name')
                           . '] changed: ' . $this->pageName()));

        $this->_page['page_text'] = $newtext;
    }

    function pageID()
    {
        return isset($this->_page['page_id']) ? $this->_page['page_id'] : '';
    }

    function pageName()
    {
        return isset($this->_page['page_name'])
            ? $this->_page['page_name']
            : '';
    }

    function getText()
    {
        return isset($this->_page['page_text'])
            ? $this->_page['page_text']
            : '';
    }

    function versionCreated()
    {
        return isset($this->_page['version_created'])
            ? $this->_page['version_created']
            : '';
    }

    function hits()
    {
        return !empty($this->_page['page_hits'])
            ? $this->_page['page_hits']
            : 0;
    }

    function changeLog()
    {
        return $this->_page['change_log'];
    }

    function version()
    {
        if (isset($this->_page['page_majorversion']) &&
            isset($this->_page['page_minorversion'])) {
            return $this->_page['page_majorversion'] . '.' .
                $this->_page['page_minorversion'];
        } else {
            return '';
        }
    }

    function diff($version)
    {
        require WICKED_TEMPLATES . '/diff/diff.inc';
    }

    /**
     * Produces a diff for this page.
     *
     * @param string $version   Previous version, or null if diffing with
     *                          `before the beginning' (empty).
     * @param string $renderer  The diff renderer.
     */
    function getDiff($version, $renderer = 'unified')
    {
        if (is_null($version)) {
            $old_page_text = '';
        } else {
            $old_page = $this->getPage($this->pageName(), $version);
            $old_page_text = $old_page->getText();
        }

        include_once 'Text/Diff.php';
        include_once 'Text/Diff/Renderer.php';
        include_once 'Text/Diff/Renderer/' . $renderer . '.php';

        $diff = new Text_Diff(explode("\n", $old_page_text),
                              explode("\n", $this->getText()));

        $class = 'Text_Diff_Renderer_' . $renderer;
        $renderer = new $class();
        return $renderer->render($diff);
    }

}
